import { initializeModalStore } from './Modal/stores.js';
import { initializeToastStore } from './Toast/stores.js';
import { initializeDrawerStore } from './Drawer/stores.js';
/**
 * Used to initialize the stores for the `Modal`, `Toast`, and `Drawer` utilities.
 *
 * @example
 * ```svelte
 * <!-- App's root +layout.svelte -->
 * <script>
 * 	import { initializeStores, Toast, Modal, Drawer } from "@skeletonlabs/skeleton";
 *
 * 	initializeStores();
 * </script>
 *
 * <Toast />
 * <Modal />
 * <Drawer />
 * ```
 */
export function initializeStores() {
    initializeModalStore();
    initializeToastStore();
    initializeDrawerStore();
}
